# Copyright 2018 Meltwater
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Let users know if they're missing any of our hard dependencies
hard_dependencies = ("gremlin_python",)
missing_dependencies = []

for dependency in hard_dependencies:
    try:
        __import__(dependency)
    except ImportError as e:
        missing_dependencies.append(dependency)

if missing_dependencies:
    raise ImportError(
        "Missing required dependencies {0}".format(missing_dependencies))
del hard_dependencies, dependency, missing_dependencies

from .pandify import to_df, toDF

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



from jugri import to_df, toDF
import numpy as np
import pandas as pd
import unittest
from gremlin_python.process.graph_traversal import GraphTraversal
from gremlin_python.structure.graph import Vertex, Path


def wrap_content_as_traversal(*content):
    mockTraversal = GraphTraversal.__new__(GraphTraversal)
    mockTraversal.toList = lambda: list(content)
    return mockTraversal


class TestPandification(unittest.TestCase):

    def testNestedFields(self):
        traversal_result = [{"T.id":"T1","T.label":"node" },
                      {"T.id":"T2","T.label":"node" , "nested": {"field1": [0]}},
                      {"T.id":"T3","T.label":"node", "nested": {"field1": [1]}},
                      {"T.id":"T4","T.label":"node" }]
        df = to_df(traversal_result)
        test_df = pd.DataFrame(columns=['T.id', 'T.label', 'nested.field1'],
                               data=[['T1', 'node', np.nan],
                                     ['T2', 'node', 0.0],
                                     ['T3', 'node', 1.0],
                                     ['T4', 'node', np.nan]],
                               index=[0, 1, 2, 3])

        self.assertTrue(test_df.equals(df)
                             , "Dictionary with nested field does not transform properly.")

    def testEmptyResult(self):
        traversal_result = []
        df = to_df(traversal_result)
        self.assertTrue(pd.DataFrame().equals(df), "Empty result does not yield empty DataFrame.")

    def testToListCall(self):
        traversal = wrap_content_as_traversal(Vertex("v1"), Vertex("v2"), Vertex("v3"))
        df = to_df(traversal)
        self.assertEqual(len(df.index), 3, "DataFrame is not populated properly.")

    def testColumns(self):
        traversal = wrap_content_as_traversal(Vertex("v1"), Vertex("v2"), Vertex("v3"))
        df = to_df(traversal)
        self.assertListEqual(df.columns.values.tolist(), ['id', 'label'], "Incorrect field names extracted.")

    def testProfiling(self):
        data = [{'@type': 'g:TraversalMetrics',
                 '@value': {'dur': 0.8397500000000001,
                            'metrics': [{'@type': 'g:Metrics',
                                         '@value': {'dur': 0.210785,
                                                    'counts': {'traverserCount': 1, 'elementCount': 1},
                                                    'name': 'NeptuneGraphQueryStep(Vertex)',
                                                    'annotations': {'percentDur': 25.10092289371837},
                                                    'id': '8.0.0()'}},
                                        {'@type': 'g:Metrics',
                                         '@value': {'dur': 0.628965,
                                                    'counts': {'traverserCount': 1, 'elementCount': 1},
                                                    'name': 'PropertyMapStep(value)',
                                                    'annotations': {'percentDur': 74.89907710628164},
                                                    'id': '4.0.0()'}}]}}]
        df = to_df(data)
        self.assertAlmostEqual(df.iloc[0,0],25.10092289371837, 5,"DataFrame format is not correct.")
        self.assertAlmostEqual(df.loc[1,"annotations.percentDur"], 74.89908,4, "Profile data not correctly flattened.")

    def testNoProfiling(self):
        data = [{'@type': 'g:TraversalMetrics',
                 '@value': {'dur': 0.8397500000000001,
                            'metrics': [{'@type': 'g:Metrics',
                                         '@value': {'dur': 0.210785,
                                                    'counts': {'traverserCount': 1, 'elementCount': 1},
                                                    'name': 'NeptuneGraphQueryStep(Vertex)',
                                                    'annotations': {'percentDur': 25.10092289371837},
                                                    'id': '8.0.0()'}},
                                        {'@type': 'g:Metrics',
                                         '@value': {'dur': 0.628965,
                                                    'counts': {'traverserCount': 1, 'elementCount': 1},
                                                    'name': 'PropertyMapStep(value)',
                                                    'annotations': {'percentDur': 74.89907710628164},
                                                    'id': '4.0.0()'}}]}}]
        df = to_df(data, detect_profiling=False)
        self.assertEqual(df.loc[0,'@type'],'g:TraversalMetrics',"DataFrame format is not correct.")

    def testDictList(self):
        data = [{'Cluster': [
                             '_id',
                             'cluster.prop1.value',
                             '_label'],
                 'Organization': ['org.prop2.value',
                                  '_id',
                                  '_label'],
                 'Address': ['addr.prop1.value',
                             'addr.prop2.value',
                             '_label',
                             '_id'],
                 'Job': [
                         '_id',
                         '_label'],
                 'Person': ['cluster.prop1.value',
                            'cluster.prop2.value',
                            'cluster.prop3.value',
                            '_id',
                            '_label'],
                 'Source': [
                            '_id'],
                'Nothing': []
        }]
        df = to_df(data)
        self.assertEqual(df['Nothing'].values[0], None)
        self.assertEqual(df['Source'].values[0], '_id')
        self.assertListEqual(df['Job'].values[0], [
            '_id',
            '_label'])





from gremlin_python.structure.graph import Element, Path
from gremlin_python.process.graph_traversal import GraphTraversal
import pandas as pd
import collections
import logging

logger = logging.getLogger(__name__)


def _flatten(d, parent_key='', sep='_'):
    """
    Flatten nested fields using recursive calls.
    :param d: dictionary
    :param parent_key: current key
    :param sep: key separator
    :return: flattened dictionary
    """
    items = []
    for k, v in d.items():
        new_key = parent_key + sep + str(k) if parent_key else k
        if isinstance(v, collections.MutableMapping):
            items.extend(_flatten(v, new_key, sep=sep).items())
        else:
            items.append((new_key, v))
    return dict(items)


def _get_singular(value):
    if type(value) is list:
        lv = len(value)
        if lv == 1:
            return value[0]
        elif lv == 0:
            None
        else:
            return value
    else:
        return value


def toDF(*args, **kwargs):
    raise DeprecationWarning("to_df should be used instead.")
    to_df(args, kwargs)


def to_df(gremlin_traversal, key_value_pairs=False, flatten_dict=True, auto_cardinality=True, detect_profiling=True):
    # type: (bool, bool, bool, bool) -> pd.DataFrame
    """
    Converts a Gremlin Traversal to a Pandas DataFrame. It expects a traversal or a list of traversal results.
    :param gremlin_traversal: A gremlinpython graph traversal (e.g. g.V().limit(5))
            or a list (e.g. g.V().limit(5).toList())
    :param keep_first_only: Deprecated since v0.3. Conversion is automatic and can be turned off
            using the auto_cardinality option.
    :param key_value_pairs: Set it to True when a map is returned. The DataFrame will have only two columns: the key and
            the values.
    :param flatten_dict: Set it to True if you have nested fields in your results. The field name will automatically
            become the "." concatenated hierarchy of the names (e.g. start.date.month, end.date.year, etc.)
    :param auto_cardinality: Set it to False for better performance. Singe element arrays will be automatically
            converted into the first element.
    :param detect_profiling: Automatically detect and prettify profiling results.
    :return: Pandas DataFrame
    """
    if type(gremlin_traversal) is GraphTraversal:
        gremlin_traversal = gremlin_traversal.toList()
    if len(gremlin_traversal) == 0:
        return pd.DataFrame()

    logger.debug("Type of first element: {}".format(type(gremlin_traversal[0])))

    if type(gremlin_traversal[0]) is dict:
        if detect_profiling and "@type" in gremlin_traversal[0].keys() and gremlin_traversal[0]["@type"] == "g:TraversalMetrics":
            df = pd.DataFrame([_flatten(_["@value"], sep='.') for _ in gremlin_traversal[0]["@value"]["metrics"]])
        else:
            if flatten_dict:
                gremlin_traversal = [_flatten(_, sep='.') for _ in gremlin_traversal]
            if key_value_pairs:
                df = pd.DataFrame(data={"value": [list(_.values())[0] for _ in gremlin_traversal]},
                                  index=[list(_.keys())[0] for _ in gremlin_traversal])
            else:
                df = pd.DataFrame(gremlin_traversal)
    elif isinstance(gremlin_traversal[0], Element):
        df = pd.DataFrame([_.__dict__ for _ in gremlin_traversal])
    elif type(gremlin_traversal[0]) is Path:
        df = pd.DataFrame([dict(enumerate(_)) for _ in gremlin_traversal])
    else:
        df = pd.DataFrame(gremlin_traversal)
    if auto_cardinality:
        return df.applymap(_get_singular)
    else:
        return df


#!/usr/bin/env python
import versioneer
from setuptools import setup, find_packages

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()


def read_all(filename):
    """
    Read file line by line
    """
    with open(filename, "r") as myfile:
        yield myfile.readline()


dependencies = list(read_all('requirements.txt'))


setup(name='jugri',
      version=versioneer.get_version(),
      cmdclass=versioneer.get_cmdclass(),
      description=long_description,
      description_content_type='text/markdown',
      author='Laszlo Velinszky',
      author_email='laszlo.velinszky@meltwater.com',
      url='https://github.com/meltwater/jugri',
      license='ASL2',
      packages=find_packages(),
      setup_requires=["pytest-runner"] + dependencies,
      tests_requires=["pytest"] + dependencies,
      test_suite="tests"
)
